/**
 * 
 */
package org.processmining.models.simpleprecedencediagram;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import javax.swing.SwingConstants;

import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.directed.AbstractDirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraph;
import org.processmining.models.graphbased.directed.DirectedGraphEdge;
import org.processmining.models.graphbased.directed.DirectedGraphElement;
import org.processmining.models.graphbased.directed.DirectedGraphNode;
import org.processmining.models.simpleprecedencediagram.elements.SPDArcElement;
import org.processmining.models.simpleprecedencediagram.elements.SPDNodeElement;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version Mar 16, 2009
 */
public class SPDImpl extends AbstractDirectedGraph<SPDNode, SPDEdge<? extends SPDNode, ? extends SPDNode>> implements
		SPD {

	protected final Set<SPDNode> nodes;
	protected final Set<SPDEdge<SPDNode, SPDNode>> arcs;

	public SPDImpl(String label) {
		super();
		nodes = new LinkedHashSet<SPDNode>();
		arcs = new LinkedHashSet<SPDEdge<SPDNode, SPDNode>>();
		getAttributeMap().put(AttributeMap.PREF_ORIENTATION, SwingConstants.WEST);
		getAttributeMap().put(AttributeMap.LABEL, label);
	}

	@Override
	protected SPDImpl getEmptyClone() {
		return new SPDImpl(getLabel());
	}

	protected Map<? extends DirectedGraphElement, ? extends DirectedGraphElement> cloneFrom(
			DirectedGraph<SPDNode, SPDEdge<? extends SPDNode, ? extends SPDNode>> graph) {
		HashMap<DirectedGraphElement, DirectedGraphElement> mapping = new HashMap<DirectedGraphElement, DirectedGraphElement>();

		for (SPDNode a : graph.getNodes()) {
			mapping.put(a, addNode(a.getLabel()));
		}

		getAttributeMap().clear();
		AttributeMap map = graph.getAttributeMap();
		for (String key : map.keySet()) {
			getAttributeMap().put(key, map.get(key));
		}
		return mapping;
	}

	/**
	 * NODE
	 */
	public synchronized SPDNode addNode(String label) {
		SPDNode node = new SPDNodeElement(label, this);
		nodes.add(node);
		graphElementAdded(node);
		return node;
	}

	public synchronized SPDNode removeNode(SPDNode spdNode) {
		removeSurroundingEdges(spdNode);
		return removeNodeFromCollection(nodes, spdNode);
	}

	public void removeNode(DirectedGraphNode cell) {
		if (cell instanceof SPDNode) {
			removeNode((SPDNode) cell);
		}
	}

	public synchronized Set<SPDNode> getNodes() {
		Set<SPDNode> newNodes = new HashSet<SPDNode>();
		newNodes.addAll(nodes);
		return newNodes;
	}

	/**
	 * ARC
	 */
	public synchronized SPDEdge<SPDNode, SPDNode> addArc(SPDNode source, SPDNode target) {
		return addArc(source, target, null);
	}

	public synchronized SPDEdge<SPDNode, SPDNode> addArc(SPDNode source, SPDNode target, String label) {
		checkAddEdge(source, target);
		SPDArcElement a = new SPDArcElement(source, target);
		if (arcs.add(a)) {
			graphElementAdded(a);
			return a;
		} else {
			for (SPDEdge<SPDNode, SPDNode> existing : arcs) {
				if (existing.equals(a)) {
					if (label != null) {
						existing.getAttributeMap().put(AttributeMap.LABEL, label);
					}
					return existing;
				}
			}
		}
		assert (false);
		return null;
	}

	public synchronized SPDEdge<SPDNode, SPDNode> removeArc(SPDNode source, SPDNode target) {
		return removeFromEdges(source, target, arcs);
	}

	public SPDEdge<SPDNode, SPDNode> removeArc(SPDEdge<? extends SPDNode, ? extends SPDNode> edge) {
		return removeFromEdges(edge.getSource(), edge.getTarget(), arcs);
	}

	@SuppressWarnings("unchecked")
	public void removeEdge(DirectedGraphEdge edge) {
		if (edge instanceof SPDArcElement) {
			arcs.remove(edge);
		} else {
			assert (false);
		}
		graphElementRemoved(edge);
	}

	public synchronized SPDEdge<SPDNode, SPDNode> getArc(SPDNode source, SPDNode target) {
		Collection<SPDEdge<SPDNode, SPDNode>> set = getEdges(source, target, arcs);
		return (set.isEmpty() ? null : set.iterator().next());
	}

	public synchronized Set<SPDEdge<? extends SPDNode, ? extends SPDNode>> getEdges() {
		Set<SPDEdge<? extends SPDNode, ? extends SPDNode>> edges = new HashSet<SPDEdge<? extends SPDNode, ? extends SPDNode>>();
		edges.addAll(arcs);
		return edges;
	}
}
