/**
 * 
 */
package org.processmining.models.fuzzyperformancediagram;

import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.models.graphbased.directed.DirectedGraph;

/**
 * @author arya
 * @email arya.adriansyah@gmail.com
 * @version Mar 19, 2009
 */
public interface FPD extends DirectedGraph<FPDNode, FPDEdge<? extends FPDNode, ? extends FPDNode>> {

	/**
	 * Get label of an FPD
	 */
	public String getLabel();

	/**
	 * add a new FPD node with a certain label
	 * 
	 * @param label
	 * @return
	 */
	public FPDNode addNode(String label);

	/**
	 * Remove an FPDNode from an FPD
	 * 
	 * @param fpdNodeElement
	 * @return
	 */
	public FPDNode removeNode(FPDNode fpdNodeElement);

	/**
	 * Get a set of nodes which refer to eventClass
	 * 
	 * @param eventClass
	 * @return
	 */
	public Set<FPDNode> getNodes(XEventClass eventClass);

	// ARCS
	/**
	 * add new arc between source and target node
	 * 
	 * @param source
	 * @param target
	 * @return
	 */
	public FPDEdge<FPDNode, FPDNode> addArc(FPDNode source, FPDNode target);

	/**
	 * remove an arc between source and target FPDNode
	 * 
	 * @param source
	 * @param target
	 * @return
	 */
	public FPDEdge<FPDNode, FPDNode> removeArc(FPDNode source, FPDNode target);

	/**
	 * Get an arc between source and target node. Return null if there is no
	 * node
	 * 
	 * @param source
	 * @param target
	 * @return
	 */
	public FPDEdge<FPDNode, FPDNode> getArc(FPDNode source, FPDNode target);

}
